# -*- coding:utf-8 -*-
# This script will download all the 10-K, 10-Q and 8-K
# provided that of company symbol and its cik code.
from __future__ import print_function  # Compatibility with Python 2

import requests
import os
import errno
from bs4 import BeautifulSoup
import datetime

DEFAULT_DATA_PATH = os.path.abspath(os.path.join(
    os.path.dirname(__file__), '..', 'SEC-Edgar-Data'))


class SecCrawler(object):

    def __init__(self, data_path=DEFAULT_DATA_PATH):
        self.data_path = data_path
        print("Path of the directory where data will be saved: " + self.data_path)

    def __repr__(self):
        return "SecCrawler(data_path={0})".format(self.data_path)

    def _make_directory(self, company_code, cik, priorto, filing_type):
        # Making the directory to save comapny filings
        path = os.path.join(self.data_path, company_code, cik, filing_type)

        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except OSError as exception:
                if exception.errno != errno.EEXIST:
                    raise

    def _save_in_directory(self, company_code, cik, priorto, filing_type, docs):
        # Save every text document into its respective folder
        for (url, doc_name) in docs:
            r = requests.get(url)
            data = r.text
            path = os.path.join(self.data_path, company_code, cik,
                                filing_type, doc_name)

            with open(path, "ab") as f:
                f.write(data.encode('ascii', 'ignore'))

    @staticmethod
    def _create_document_list(data):
        # parse fetched data using beatifulsoup
        # Explicit parser needed
        soup = BeautifulSoup(data, features='html.parser')
        # store the link in the list
        link_list = [link.string for link in soup.find_all('filinghref')]

        print("Number of files to download: {0}".format(len(link_list)))
        print("Starting download...")

        # List of url to the text documents
        txt_urls = [link[:link.rfind("-")] + ".txt" for link in link_list]
        # List of document doc_names
        doc_names = [url.split("/")[-1] for url in txt_urls]

        return list(zip(txt_urls, doc_names))

    @staticmethod
    def _sanitize_date(date):
        if isinstance(date, datetime.datetime):
            return date.strftime("%Y%m%d")
        elif isinstance(date, str):
            if len(date) != 8:
                raise TypeError('Date must be of the form YYYYMMDD')
        elif isinstance(date, int):
            if date < 10**7 or date > 10**8:
                raise TypeError('Date must be of the form YYYYMMDD')

    def _fetch_report(self, company_code, cik, priorto, count, filing_type):
        priorto = self._sanitize_date(priorto)
        self._make_directory(company_code, cik, priorto, filing_type)

        # generate the url to crawl
        base_url = "http://www.sec.gov/cgi-bin/browse-edgar"
        params = {'action': 'getcompany', 'owner': 'exclude', 'output': 'xml',
                  'CIK': cik, 'type': filing_type, 'dateb': priorto, 'count': count}
        print("started {filing_type} {company_code}".format(
            filing_type=filing_type, company_code=company_code))
        r = requests.get(base_url, params=params)
        data = r.text

        # get doc list data
        docs = self._create_document_list(data)

        try:
            self._save_in_directory(
                company_code, cik, priorto, filing_type, docs)
        except Exception as e:
            print(str(e))  # Need to use str for Python 2.5

        print("Successfully downloaded all the files")

    def filing_10Q(self, company_code, cik, priorto, count):
        self._fetch_report(company_code, cik, priorto, count, '10-Q')

    def filing_10K(self, company_code, cik, priorto, count):
        self._fetch_report(company_code, cik, priorto, count, '10-K')

    def filing_8K(self, company_code, cik, priorto, count):
        self._fetch_report(company_code, cik, priorto, count, '8-K')

    def filing_13F(self, company_code, cik, priorto, count):
        self._fetch_report(company_code, cik, priorto, count, '13-F')

    def filing_SD(self, company_code, cik, priorto, count):
        self._fetch_report(company_code, cik, priorto, count, 'SD')

    def filing_4(self, company_code, cik, priorto, count):
        self._fetch_report(company_code, cik, priorto, count, '4')



import pytest


class TestCrawler(object):
    @pytest.mark.xfail(raises=TypeError)
    def test_10Q_requires_args():
        return crawler.filing_10Q()

    @pytest.mark.xfail(raises=TypeError)
    def test_10K_requires_args():
        return crawler.filing_10K()

    @pytest.mark.xfail(raises=TypeError)
    def test_SD_requires_args():
        return crawler.filing_SD()

    @pytest.mark.xfail(raises=TypeError)
    def test_8K_requires_args():
        return crawler.filing_8K()

    @pytest.mark.xfail(raises=TypeError)
    def test_13F_requires_args():
        return crawler.filing_13F()
    
    @pytest.mark.xfail(raises=TypeError)
    def test_4_requires_args():
        return crawler.filing_4()




# -*- coding:utf-8 -*-
import time
from crawler import SecCrawler


def test():
    t1 = time.time()
    # file containig company name and corresponding cik codes
    seccrawler = SecCrawler()

    company_code_list = list()   # company code list
    cik_list = list()            # cik code list
    date_list = list()           # pror date list
    count_list = list()

    try:
        with open("data.txt", "r") as f:
            # get the comapny  quotes and cik number from the file.
            for columns in (raw.strip().split() for raw in f):
                company_code_list.append(columns[0])
                cik_list.append(columns[1])
                date_list.append(columns[2])
                count_list.append(columns[3])
    except:
        print("No input file Found")

    # call different  API from the crawler
    for i in range(1, len(cik_list)):
        seccrawler.filing_SD(company_code_list[i], cik_list[i],
                             date_list[i], count_list[i])
        seccrawler.filing_10K(company_code_list[i], cik_list[i],
                              date_list[i], count_list[i])
        seccrawler.filing_8K(company_code_list[i], cik_list[i],
                             date_list[i], count_list[i])
        seccrawler.filing_10Q(company_code_list[i], cik_list[i],
                              date_list[i], count_list[i])

    t2 = time.time()
    print("Total Time taken: {0}".format(t2 - t1))


if __name__ == '__main__':
    test()



# -*- coding:utf-8 -*-
import sys

from setuptools import setup, find_packages
from setuptools.command.install import install

SUPPORTED_VERSIONS = ['2.7', '3.5', '3.6', 'PyPy', ]

if sys.version_info <= (2, 4):
    version = '.'.join([str(x) for x in sys.version_info[:3]])
    suppored_versions = ', '.join(SUPPORTED_VERSIONS)
    print('Version {version} is not supported. Supported versions are {supported_versions}'
          .format(version=version, supported_versions=SUPPORTED_VERSIONS)
          )
    sys.exit(1)


def parse_requirements(filename):

    with open(filename) as f:
        required = f.read().splitlines()
        return required


setup(
    name='SECEdgar',
    version='0.1.4',
    packages=find_packages(),
    package_dir={'SECEdgar': 'SECEdgar'},
    url='https://github.com/rahulrrixe/SEC-Edgar',
    license='Apache License (2.0)',
    author='Rahul Ranjan',
    author_email='rahul.rrixe@gmail.com',
    description="""SEC-Edgar implements a basic Sphinx crwaler for downloading the 
                 filings. It provides an interface to extract the filing from 
                 the SEC.gov site. You might find it most useful for tasks 
                 involving automated data collection of filings from SEC.gov""",
    entry_points='''
            [console_scripts]
            ''',
    cmdclass={
        'install': install,
    },
    install_requires=parse_requirements('requirements.txt'),
    keywords=['SEC', 'Edgar', 'Crawler', 'filings'],
    tests_require=parse_requirements('requirements.txt'),
    classifiers=[
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: PyPy']
)
